<?php

/*
Plugin Name: Passport
Plugin URI:
Description: A WordPress plugin that allows users to login by authenticating with Passport via OAuth 2.0.
Version: 0.1
Author: Derek Klatt
Author URI: http://www.inversoft.com
Copyright [2016] [Inversoft]

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

session_start();

Class Passport
{

    const PLUGIN_VERSION = "0.1";

    protected static $instance = NULL;

    public static function get_instance()
    {
        NULL === self::$instance and self::$instance = new self;
        return self::$instance;
    }

    private $settings = array(
        'passport_login_redirect' => 'home_page',
        'passport_login_redirect_page' => 0,
        'passport_login_redirect_url' => '',
        'passport_suppress_welcome_email' => 0,
        'passport_logout_inactive_users' => 0,
        'passport_new_user_role' => '',
        'passport_api_id' => '',
        'passport_api_secret' => '',
        'passport_api_key' => '',
        'passport_frontend_url' => '',
        'passport_backend_url' => '',
        'passport_http_util_verify_ssl' => 1,
        'passport_restore_default_settings' => 0,
        'passport_delete_settings_on_uninstall' => 0,
    );

    function __construct()
    {
        add_action('init', array($this, 'init'));
    }

    function init()
    {
        add_filter('query_vars', array($this, 'passport_qvar_triggers'));
        add_action('template_redirect', array($this, 'passport_qvar_handlers'));
        add_action('admin_enqueue_scripts', array($this, 'passport_init_admin_scripts_styles'));
        add_action('admin_menu', array($this, 'passport_settings_page'));
        add_action('admin_init', array($this, 'passport_register_settings'));
        $plugin = plugin_basename(__FILE__);
        add_filter("plugin_action_links_$plugin", array($this, 'passport_settings_link'));
        add_action('wp_logout', array($this, 'passport_end_logout'));
        add_action('wp_ajax_passport_logout', array($this, 'passport_logout_user'));
        add_filter('login_url',
            function () {
                $params = array('response_type' => 'code', 'client_id' => get_option('passport_api_id'), 'redirect_uri' => site_url());
                return get_option('passport_frontend_url') . "/oauth2/authorize?" . http_build_query($params);
            });
        add_filter('allowed_redirect_hosts',
            function ($content) {
                $content[] = parse_url(get_option('passport_frontend_url'), PHP_URL_HOST);
                return $content;
            });
    }

    function passport_init_admin_scripts_styles()
    {
        wp_enqueue_style('passport-style', plugin_dir_url(__FILE__) . 'passport.css', array());
    }

    function passport_settings_link($links)
    {
        $settings_link = "<a href='options-general.php?page=passport.php'>Settings</a>";
        array_unshift($links, $settings_link);
        return $links;
    }

    function passport_qvar_triggers($vars)
    {
        $vars[] = 'code';
        $vars[] = 'error_description';
        $vars[] = 'error_message';
        return $vars;
    }

    private function get_oauth_token()
    {
        $params = array(
            'grant_type' => 'authorization_code',
            'client_id' => get_option('passport_api_id'),
            'code' => $_GET['code'],
            'redirect_uri' => rtrim(site_url(), '/')
        );
        $url_params = http_build_query($params);

        $url = get_option('passport_frontend_url') . "/oauth2/token?" . $url_params;
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_POST, 1);

        $result = curl_exec($curl);

        $result_obj = json_decode($result, true);
        $access_token = $result_obj['access_token'];
        $expires_in = $result_obj['expires_in'];
        $expires_at = time() + $expires_in;
        if (!$access_token || !$expires_in) {
            $this->passport_end_login("Sorry, we couldn't log you in. Malformed access token result detected. Please notify the admin or try again later.");
        } else {
            $_SESSION['PASSPORT']['ACCESS_TOKEN'] = $access_token;
            $_SESSION['PASSPORT']['USER_ID'] = $result_obj['userId'];
            $_SESSION['PASSPORT']['EXPIRES_IN'] = $expires_in;
            $_SESSION['PASSPORT']['EXPIRES_AT'] = $expires_at;
            return true;
        }
    }

    private function get_oauth_identity()
    {
        $url = get_option('passport_backend_url') . "/api/user/" . $_SESSION['PASSPORT']['USER_ID'];
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_HTTPHEADER, array('Authorization:' . get_option('passport_api_key')));
        $result = curl_exec($curl);
        $result_obj = json_decode($result, true);
        $user = $result_obj['user'];
        $registrations = $user['registrations'];
        $oauth_identity = array();
        $oauth_identity['id'] = $user['id'];
        $oauth_identity['email'] = $user['email'];
        $oauth_identity['username'] = $user['username'];

        foreach ($registrations as $app) {
            if ($app['applicationId'] == get_option('passport_api_id')) {
                $roles = $app['roles'];
            }
        }

        if (sizeof($roles) > 1) {
            if ($roles['administrator']) {
                $oauth_identity['role'] = "administrator";
            } elseif ($roles["editor"]) {
                $oauth_identity['role'] = "editor";
            } elseif ($roles['author']) {
                $oauth_identity['role'] = "author";
            } else {
                $oauth_identity['role'] = "contributor";
            }
        } elseif (sizeof($roles == 1)) {
            $oauth_identity['role'] = $roles[0];
        }

        if (!$oauth_identity['id']) {
            $this->passport_end_login("Sorry, we couldn't log you in. User identity was not found. Please notify the admin or try again later.");
        }
        return $oauth_identity;
    }

    function passport_qvar_handlers()
    {
        if (get_query_var('code') || get_query_var('error_description') || get_query_var('error_message')) {
            if (!$_SESSION['PASSPORT']['LAST_URL']) {
                $redirect_url = esc_url($_GET['redirect_to']);
                if (!$redirect_url) {
                    $redirect_url = strtok($_SERVER['HTTP_REFERER'], "?");
                }
                $_SESSION['PASSPORT']['LAST_URL'] = $redirect_url;
            }

            if (isset($_GET['error_description'])) {
                $this->passport_end_login($_GET['error_description']);
            } elseif (isset($_GET['error_message'])) {
                $this->passport_end_login($_GET['error_message']);
            } elseif (isset($_GET['code'])) {
                $this->get_oauth_token();
                $oauth_identity = $this->get_oauth_identity();
                $this->passport_login_user($oauth_identity);
            } else {
                if ((empty($_SESSION['PASSPORT']['EXPIRES_AT'])) || (time() > $_SESSION['PASSPORT']['EXPIRES_AT'])) {
                    $this->passport_clear_login_state();
                }
            }
            $this->passport_end_login("Sorry, we couldn't log you in. The authentication flow terminated in an unexpected way. Please notify the admin or try again later.");
        }
    }

    private function passport_match_wordpress_user($oauth_identity)
    {
        global $wpdb;
        $usermeta_table = $wpdb->usermeta;
        $query_string = "SELECT $usermeta_table.user_id FROM $usermeta_table WHERE $usermeta_table.meta_key = 'passport_identity' AND $usermeta_table.meta_value = '" . $oauth_identity['id'] . "'";
        $query_result = $wpdb->get_var($query_string);
        $user = get_user_by('id', $query_result);
        return $user;
    }

    private function register()
    {
        global $wpdb;

        if ($_SESSION["PASSPORT"]["USER_ID"] != "") {
            $id = uniqid('', true);
            $password = wp_generate_password();
        }

        if ($_SESSION["PASSPORT"]["USER_ID"] == "") {
            $id = $_POST['identity'];
            $password = $_POST['password'];
        }

        if ($_SESSION["PASSPORT"]["USER_NAME"] != "") {
            $username = $_SESSION["PASSPORT"]["USER_NAME"];
        } else {
            $username = "user" . $id;
        }

        if ($_SESSION["PASSPORT"]["EMAIL"] != "") {
            $email = $_SESSION["PASSPORT"]["EMAIL"];
        }

        $user_id = wp_create_user($id, $password, $email);

        if (is_wp_error($user_id)) {
            $_SESSION["PASSPORT"]["RESULT"] = $user_id->get_error_message();
            header("Location: " . esc_url($_GET['redirect_to']));
            exit;
        }

        $update_username_result = $wpdb->update($wpdb->users, array('user_login' => $username, 'user_nicename' => $username, 'display_name' => $username), array('ID' => $user_id));
        $update_nickname_result = update_user_meta($user_id, 'nickname', $username);

        $role = get_option('passport_new_user_role');
        $update_role_result = wp_update_user(array('ID' => $user_id, 'role' => $role));

        if ($update_username_result == false || $update_nickname_result == false) {
            $_SESSION["PASSPORT"]["RESULT"] = "Could not rename the username during registration. Please contact an admin or try again later.";
            header("Location: " . esc_url($_GET['redirect_to']));
            exit;
        } elseif ($update_role_result == false) {
            $_SESSION["PASSPORT"]["RESULT"] = "Could not assign default user role during registration. Please contact an admin or try again later.";
            header("Location: " . esc_url($_GET['redirect_to']));
            exit;
        } else {
            $this->passport_link_account($user_id);
            $creds = array();
            $creds['user_login'] = $username;
            $creds['user_password'] = $password;
            $creds['remember'] = true;
            $user = wp_signon($creds, false);
            if (!get_option('passport_suppress_welcome_email')) {
                wp_new_user_notification($user_id, $password);
            }
            $_SESSION["PASSPORT"]["RESULT"] = "You have been registered successfully!";
            header("Location: " . esc_url($_GET['redirect_to']));
            exit;
        }
    }

    private function passport_login_user($oauth_identity)
    {
        $_SESSION["PASSPORT"]["USER_NAME"] = $oauth_identity["username"];
        $_SESSION["PASSPORT"]["EMAIL"] = $oauth_identity["email"];
        $matched_user = $this->passport_match_wordpress_user($oauth_identity);
        if ($matched_user) {
            $user_id = $matched_user->ID;
            $user_login = $matched_user->user_login;
            wp_set_current_user($user_id, $user_login);
            wp_set_auth_cookie($user_id);
            do_action('wp_login', $user_login, $matched_user);
            $this->passport_end_login("Logged in successfully!");
        }
        if (is_user_logged_in()) {
            global $current_user;
            get_currentuserinfo();
            $user_id = $current_user->ID;
            $this->passport_link_account($user_id);
            $this->passport_end_login("Your account was linked successfully with your third party authentication provider.");
        }
        if (!is_user_logged_in() && !$matched_user) {
            update_option('passport_new_user_role', $oauth_identity['role']);
            $this->register();
        }
        $this->passport_end_login("Sorry, we couldn't log you in. The login flow terminated in an unexpected way. Please notify the admin or try again later.");
    }

    private function passport_end_login($msg)
    {
        $last_url = $_SESSION["PASSPORT"]["LAST_URL"];
        unset($_SESSION["PASSPORT"]["LAST_URL"]);
        $_SESSION["PASSPORT"]["RESULT"] = $msg;
        $this->passport_clear_login_state();
        $redirect_method = get_option("passport_login_redirect");
        $redirect_url = "";
        switch ($redirect_method) {
            case "home_page":
                $redirect_url = site_url();
                break;
            case "last_page":
                $redirect_url = $last_url;
                break;
            case "specific_page":
                $redirect_url = get_permalink(get_option('passport_login_redirect_page'));
                break;
            case "admin_dashboard":
                $redirect_url = admin_url();
                break;
            case "user_profile":
                $redirect_url = get_edit_user_link();
                break;
            case "custom_url":
                $redirect_url = get_option('passport_login_redirect_url');
                break;
        }

        wp_safe_redirect($redirect_url);
        die();
    }

    function passport_logout_user()
    {
        $user = null;
        session_destroy();
        wp_logout();
    }

    function passport_end_logout()
    {
        $_SESSION["PASSPORT"]["RESULT"] = 'Logged out successfully.';
        unset($_SESSION["PASSPORT"]["LAST_URL"]);
        $redirect_url = get_option('passport_frontend_url') . "/oauth2/logout?client_id=" . get_option('passport_api_id');
        wp_safe_redirect($redirect_url);
        die();
    }

    private function passport_link_account($user_id)
    {
        if ($_SESSION['PASSPORT']['USER_ID'] != '') {
            add_user_meta($user_id, 'passport_identity', $_SESSION['PASSPORT']['USER_ID']);
        }
    }

    private function passport_clear_login_state()
    {
        unset($_SESSION["PASSPORT"]["USER_ID"]);
        unset($_SESSION["PASSPORT"]["USER_EMAIL"]);
        unset($_SESSION["PASSPORT"]["ACCESS_TOKEN"]);
        unset($_SESSION["PASSPORT"]["EXPIRES_IN"]);
        unset($_SESSION["PASSPORT"]["EXPIRES_AT"]);
    }

    function passport_register_settings()
    {
        foreach ($this->settings as $setting_name => $default_value) {
            register_setting('passport_settings', $setting_name);
        }
    }

    function passport_settings_page()
    {
        add_options_page('Passport Options', 'Passport', 'manage_options', 'Passport', array($this, 'passport_settings_page_content'));
    }

    function passport_settings_page_content()
    {
        if (!current_user_can('manage_options')) {
            wp_die(__('You do not have sufficient permissions to access this page.'));
        }
        include 'passport-settings.php';
    }
}

PASSPORT::get_instance();
?>